! This file is part of HiggsBounds
!  -KW
!******************************************************************
module theory_BRfunctions
! Note that these are the Standard Model branching ratios used internally by HiggsBounds
! NOT the Standard Model branching ratio functions available to the user
! (see access_SM.f90 for those)
!******************************************************************

 use interpolate
 use S95tables_type1
 
 implicit none
 
 !table type 1-----------------------------	
 type(table1),allocatable :: BRSM(:)
 !------------------------------------------

 double precision :: BRSMt1Mhmax,BRSMt1Mhmin
 
 contains 

 !****************************************************** 
 subroutine setup_BRSM
 ! reads in the Standard Model Branching ratios and total decay width from files
 !****************************************************** 
  use store_pathname
  implicit none
  !------------------------------------internal
  integer :: x,xbeg,xend
  character(len=100),allocatable :: filename(:)
  character(LEN=pathname_length+150) :: fullfilename
  !--------------------------------------------
 
  allocate(BRSM(2))
  
  xbeg=lbound(BRSM,dim=1)
  xend=ubound(BRSM,dim=1)
  
  allocate(filename(xbeg:xend))    
  x=xbeg-1
 
  x=x+1 
  BRSM(x)%xmin=0.8D0
  BRSM(x)%xmax=500.0D0
  BRSM(x)%sep=0.1D0 
  filename(x)='br.sm1_HDecay' 
 
  x=x+1 
  BRSM(x)%xmin=0.8D0
  BRSM(x)%xmax=500.0D0
  BRSM(x)%sep=0.1D0 
  filename(x)='br.sm2_HDecay' 
  
  ! checks we've filled the whole array
  if(x.ne.xend)then
   stop'error in setup_BRSM (a)'
  endif 
 
  ! do loop to read in S95 tables 
  do x=xbeg,xend
  
   fullfilename=trim(adjustl(pathname))//'Theory_tables/HDecay_BR-SM/' &
             &   //trim(filename(x))
   	
   call read_tabletype1(BRSM(x),3,7,fullfilename)	      
     
  enddo	 
 
  ! we want the (smallest) range of these functions
  ! initial (extreme) values
  BRSMt1Mhmin=1.0D6
  BRSMt1Mhmax=0.0D0

  do x=xbeg,xend 
    if(BRSM(x)%xmax.gt.BRSMt1Mhmax)BRSMt1Mhmax=BRSM(x)%xmax 
    if(BRSM(x)%xmin.lt.BRSMt1Mhmin)BRSMt1Mhmin=BRSM(x)%xmin 
  enddo

  deallocate(filename)

 end subroutine setup_BRSM
 
 !****************************************************** 
 subroutine out_of_BRSM_range(funcname)
 !******************************************************
  implicit none 
  character(LEN=*), intent(in) :: funcname
  
  write(*,*)'The SM Higgs mass given as the'
  write(*,*)'argument to function '//funcname//' should be'
  write(*,*)'between',BRSM(1)%xmin,'and',BRSM(1)%xmax
  write(*,*)'(in units of GeV).'

  stop 'error in SM function: argument out of range (see standard output for more info)'

 end subroutine out_of_BRSM_range
 !****************************************************** 
 function BRSM_HWW(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_HWW
   
  ! WW is in br.sm2_HDecay, column 5 (first column is Higgs mass)
  call interpolate_1D(x,BRSM(2),5-1,interpol)
  if(interpol.lt.-1.0D-7)call out_of_BRSM_range('BRSM_HWW')
  
  BRSM_HWW=interpol
 
 end function BRSM_HWW
 
 !****************************************************** 
 function BRSM_HZZ(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_HZZ
   
  ! ZZ is in br.sm2_HDecay, column 6 (first column is Higgs mass)
  call interpolate_1D(x,BRSM(2),6-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_HZZ: x out of range'
  
  BRSM_HZZ=interpol
 
 end function BRSM_HZZ
 !****************************************************** 
 function BRSM_Hbb(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_Hbb

  ! bb is in br.sm1_HDecay, column 2 (first column is Higgs mass)
  call interpolate_1D(x,BRSM(1),2-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_Hbb: x out of range'

  BRSM_Hbb=interpol
 
 end function BRSM_Hbb 
 !****************************************************** 
 function BRSM_Htautau(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_Htautau
  
  ! tautau is in br.sm1_HDecay, column 3 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(1),3-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_Htautau: x out of range'
  
  BRSM_Htautau=interpol
 
 end function BRSM_Htautau
 !****************************************************** 
 function BRSM_Hgaga(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_Hgaga
  
  ! photon photon is in br.sm2_HDecay, column 3 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(2),3-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_Hgaga: x out of range'
  
  BRSM_Hgaga=interpol
 
 end function BRSM_Hgaga 
 !****************************************************** 
 function BRSM_Hgg(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_Hgg
  
  ! gluon gluon is in br.sm2_HDecay, column 2 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(2),2-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_Hgg: x out of range'
  
  BRSM_Hgg=interpol
 
 end function BRSM_Hgg 
 !****************************************************** 
 function BRSM_Htoptop(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_Htoptop
  
  ! top top is in br.sm1_HDecay, column 7 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(1),7-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_Htoptop: x out of range'
  
  BRSM_Htoptop=interpol
 
 end function BRSM_Htoptop 

 !****************************************************** 
 function BRSM_Hcc(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_Hcc
  
  ! charm charm is in br.sm1_HDecay, column 6 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(1),6-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_Hcc: x out of range'
  
  BRSM_Hcc=interpol
 
 end function BRSM_Hcc 
 !****************************************************** 
 function BRSM_Hss(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_Hss
  
  ! strange strange is in br.sm1_HDecay, column 5 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(1),5-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_Hss: x out of range'
  
  BRSM_Hss=interpol
 
 end function BRSM_Hss 
 !****************************************************** 
 function BRSM_Hmumu(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_Hmumu
  
  ! muon muon is in br.sm1_HDecay, column 4 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(1),4-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_Hmumu: x out of range'
  
  BRSM_Hmumu=interpol
 
 end function BRSM_Hmumu 
 !****************************************************** 
 function BRSM_HZga(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_HZga
  
  ! Z gamma is in br.sm2_HDecay, column 4 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(2),4-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_HZga: x out of range'
  
  BRSM_HZga=interpol
 
 end function BRSM_HZga  
 !****************************************************** 
 function BRSM_GammaTot(x)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: x
  double precision :: interpol
  double precision :: BRSM_GammaTot
  
  ! GammaTot is in br.sm2_HDecay, column 7 (first column is Higgs mass)  
  call interpolate_1D(x,BRSM(2),7-1,interpol)
  if(interpol.lt.-1.0D-7)stop 'error in function BRSM_GammaTot: x out of range'
  
  BRSM_GammaTot=interpol
 
 end function BRSM_GammaTot   
 !************************************************************	
 subroutine deallocate_BRSM
 !************************************************************
  implicit none
  !-----------------------------------internal
  integer x
  !-------------------------------------------
  do x=lbound(BRSM,dim=1),ubound(BRSM,dim=1)
   deallocate(BRSM(x)%dat)
  enddo 
  
  deallocate(BRSM)
 
 end subroutine deallocate_BRSM
 !************************************************************
end module theory_BRfunctions
!*******************************************************************
